#include <ESP8266WiFi.h>
#include <ESP8266HTTPClient.h>

#define SSID "Baran Network"
#define PWD "fakePassword123"

String url = "http://worldtimeapi.org/api/timezone/America/New_York";

#define SLEEP_INT 10000 // sleep interval, in ms

#define off_p 40       // psi to turn pump off
#define on_soft_p 30   // psi to turn pump on scheduled
#define on_hard_p 20   // psi to turn pump on immediately
#define hard_amt 5     // psi to pump for on_hard

#define sched_start 1 // hour to start pump schedule
#define sched_end 2   // hour to stop pump schedule - only affects when the pump turns on. it will keep running until off_p is hit, even if it crosses this time


bool pump_on = false;
bool hard_pump = false;

long startTime = 0;

void setup() {
  pinMode(D0,OUTPUT);
  digitalWrite(D0,LOW);
  Serial.begin(9600);

  Serial.println("Connecting to load time");
  WiFi.begin(SSID, PWD);
  
  while(WiFi.status() != WL_CONNECTED) {
    delay(500);
    Serial.println(WiFi.status());
  }
  Serial.println("");
  Serial.print("Connected to WiFi network with IP Address: ");
  Serial.println(WiFi.localIP());

  WiFiClient client;
  HTTPClient http;
  
  if(http.begin(client,url.c_str())){
    int httpCode = http.GET();
  
    // httpCode will be negative on error
    if (httpCode > 0) {
      // HTTP header has been send and Server response header has been handled
      Serial.printf("[HTTP] GET... code: %d\n", httpCode);
  
      // file found at server
      if (httpCode == HTTP_CODE_OK || httpCode == HTTP_CODE_MOVED_PERMANENTLY) {
        const char* payload = http.getString().c_str();
        char* timepos = strstr(payload,"\"datetime\"") + 23;
        int hour = atoi(strtok(timepos,":"));
        int minute = atoi(strtok(NULL,":"));
        int second = atoi(strtok(NULL,"."));
        startTime = second * 1000 + minute * 60000 + hour * 3600000;
        Serial.println(startTime);
      }
    }
    else {
          Serial.printf("[HTTP] GET... failed, error: %s\n", http.errorToString(httpCode).c_str());
    }
    http.end();
  } else {
    Serial.printf("[HTTP} Unable to connect\n");
  }
}

void loop() {
  int value = analogRead(A0);
  float psi = 0.0738453 * value - 19.2723;

  Serial.print("Value: ");
  Serial.print(value);
  Serial.print("  PSI: ");
  Serial.println(psi);

  if(pump_on){
    if(hard_pump && psi >= on_hard_p + hard_amt || psi >= off_p){
      // this way if hard_pump it will only pump up to the soft_p pressure and stop - then finish the pumping on schedule
      pump_on = false;
      hard_pump = false;
      digitalWrite(D0,LOW);
    }
  }
  else{
    if(psi < on_hard_p){
      pump_on = true;
      hard_pump = true;
      digitalWrite(D0,HIGH);
    }
    else if(psi < on_soft_p){
      long now = startTime + millis();
      int hour = (now % 86400000) / 3600000;
      if(sched_start < sched_end){ // does not cross over midnight
          if(hour >= sched_start && hour <= sched_end){
              pump_on = true;
              digitalWrite(D0,HIGH);
          }
      }
      else{ // does cross midnight - just do or
          if(hour >= sched_start || hour <= sched_end){
              pump_on = true;
              digitalWrite(D0,HIGH);
          }
      }
    }
  }
  
  delay(SLEEP_INT);
}
